 /*@@
   @file      HamAttenuate1.F
   @date      Thu May  2 00:56:31 2002
   @author    John Baker
   @desc 
      Attenuate the update terms with factor related to the violation
      of the Hamiltonian constraint
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"


 /*@@
   @routine    ADM_BSSN_AttenuateGauge
   @date       Thu May  2 00:57:31 2002
   @author     John Baker, Denis Pollney
   @desc 
      Reduces the update term by some factor related to the violation
      of the Hamiltonian constraint.
   @enddesc 
   @calls     
   @calledby   
   @history 
      Originally written by John Baker.
      Separated out into routines by Denis Pollney to help compiler
      optimize the loop.
   @endhistory 
   
@@*/


      subroutine ADM_BSSN_AttenuateGauge(CCTK_ARGUMENTS)
      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      integer i, j, k, nx, ny, nz
      integer stencily

      CCTK_REAL ham_atten
      CCTK_REAL twothird

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing_declare.h"

#include "macro/PHISOURCES_declare.h"
#include "macro/BSGSOURCES_declare.h"
#include "macro/TRKSOURCES_declare.h"
#include "macro/ASOURCES_declare.h"
#include "macro/GAMMASOURCES_declare.h"

#include "macro/LIEPHI_declare.h"
#include "macro/LIEBSG_declare.h"
#include "macro/LIEK_declare.h"
#include "macro/LIEA_declare.h"
#include "macro/SHIFTGAMMA_declare.h"
#include "macro/BSHAM_declare.h"

c     Grid parameters.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing.h"

      twothird = 2.0d0/3.0d0

      if (cartoon==1) then
         stencily = cctk_nghostzones(2)
      else
         stencily = 1
      end if

      do k=2,nz-1
        do j=1+stencily,ny-stencily
          do i=2,nx-1

#include "macro/BSHAM_guts.h"

            ham_atten=1./(1.+c_atten*(BSHAM_BSHAM*BSHAM_BSHAM))

            adm_bs_salp(i,j,k)=adm_bs_salp(i,j,k)*ham_atten

            adm_bs_sdtalp(i,j,k)=adm_bs_sdtalp(i,j,k)*ham_atten

            adm_bs_sbetax(i,j,k)=adm_bs_sbetax(i,j,k)*ham_atten
            adm_bs_sbetay(i,j,k)=adm_bs_sbetay(i,j,k)*ham_atten
            adm_bs_sbetaz(i,j,k)=adm_bs_sbetaz(i,j,k)*ham_atten

            adm_bs_sBx(i,j,k)=adm_bs_sBx(i,j,k)*ham_atten
            adm_bs_sBy(i,j,k)=adm_bs_sBy(i,j,k)*ham_atten
            adm_bs_sBz(i,j,k)=adm_bs_sBz(i,j,k)*ham_atten

          end do
        end do
      end do

#include "macro/BSHAM_undefine.h"
#include "macro/TRKSOURCES_undefine.h"
#include "macro/ASOURCES_undefine.h"
#include "macro/GAMMASOURCES_undefine.h"
#include "macro/PHISOURCES_undefine.h"
#include "macro/LIEPHI_undefine.h"
#include "macro/BSGSOURCES_undefine.h"
#include "macro/LIEBSG_undefine.h"
#include "macro/LIEK_undefine.h"
#include "macro/LIEA_undefine.h"
#include "macro/SHIFTGAMMA_undefine.h"

      end subroutine ADM_BSSN_AttenuateGauge


 /*@@
   @routine    ADM_BSSN_AttenuateGauge
   @date       Thu May  2 00:57:31 2002
   @author     John Baker, Denis Pollney
   @desc 
      Reduces the update term by some factor related to the violation
      of the Hamiltonian constraint.
   @enddesc 
   @calls     
   @calledby   
   @history 
      Originally written by John Baker.
      Separated out into separate routines by Denis Pollney.
   @endhistory 
   
@@*/

      subroutine ADM_BSSN_AttenuateSource(CCTK_ARGUMENTS)
      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      integer i, j, k, nx, ny, nz
      integer stencily

      CCTK_REAL ham_atten

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing_declare.h"

      CCTK_REAL twothird

#include "macro/PHISOURCES_declare.h"
#include "macro/BSGSOURCES_declare.h"
#include "macro/TRKSOURCES_declare.h"
#include "macro/ASOURCES_declare.h"
#include "macro/GAMMASOURCES_declare.h"

#include "macro/LIEPHI_declare.h"
#include "macro/LIEBSG_declare.h"
#include "macro/LIEK_declare.h"
#include "macro/LIEA_declare.h"
#include "macro/SHIFTGAMMA_declare.h"
#include "macro/BSHAM_declare.h"

c     Grid parameters.

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing.h"

      twothird = 2.0d0/3.0d0

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

      if (cartoon==1) then
         stencily = cctk_nghostzones(2)
      else
         stencily = 1
      end if

      do k=2,nz-1
        do j=1+stencily,ny-stencily
          do i=2,nx-1

#include "macro/BSHAM_guts.h"

            ham_atten=1.0D0/(1.0D0+c_atten*(BSHAM_BSHAM*BSHAM_BSHAM))

            adm_bs_sphi(i,j,k)=adm_bs_sphi(i,j,k)*ham_atten

            adm_bs_sgxx(i,j,k)=adm_bs_sgxx(i,j,k)*ham_atten
            adm_bs_sgyy(i,j,k)=adm_bs_sgyy(i,j,k)*ham_atten
            adm_bs_sgzz(i,j,k)=adm_bs_sgzz(i,j,k)*ham_atten
            adm_bs_sgxy(i,j,k)=adm_bs_sgxy(i,j,k)*ham_atten
            adm_bs_sgxz(i,j,k)=adm_bs_sgxz(i,j,k)*ham_atten
            adm_bs_sgyz(i,j,k)=adm_bs_sgyz(i,j,k)*ham_atten

            adm_bs_sK(i,j,k)=adm_bs_sK(i,j,k)*ham_atten

            adm_bs_sAxx(i,j,k)=adm_bs_sAxx(i,j,k)*ham_atten
            adm_bs_sAyy(i,j,k)=adm_bs_sAyy(i,j,k)*ham_atten
            adm_bs_sAzz(i,j,k)=adm_bs_sAzz(i,j,k)*ham_atten
            adm_bs_sAxy(i,j,k)=adm_bs_sAxy(i,j,k)*ham_atten
            adm_bs_sAxz(i,j,k)=adm_bs_sAxz(i,j,k)*ham_atten
            adm_bs_sAyz(i,j,k)=adm_bs_sAyz(i,j,k)*ham_atten

            adm_bs_sGx(i,j,k)=adm_bs_sGx(i,j,k)*ham_atten
            adm_bs_sGy(i,j,k)=adm_bs_sGy(i,j,k)*ham_atten
            adm_bs_sGz(i,j,k)=adm_bs_sGz(i,j,k)*ham_atten

          end do
        end do
      end do

#include "macro/BSHAM_undefine.h"
#include "macro/TRKSOURCES_undefine.h"
#include "macro/ASOURCES_undefine.h"
#include "macro/GAMMASOURCES_undefine.h"
#include "macro/PHISOURCES_undefine.h"
#include "macro/LIEPHI_undefine.h"
#include "macro/BSGSOURCES_undefine.h"
#include "macro/LIEBSG_undefine.h"
#include "macro/LIEK_undefine.h"
#include "macro/LIEA_undefine.h"
#include "macro/SHIFTGAMMA_undefine.h"

      end subroutine ADM_BSSN_AttenuateSource



