/*@@
  @file      RemovetrA.F
  @desc 
     Remove trace of Aij.
  @enddesc 
@@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

      subroutine ADM_BSSN_removetrA(CCTK_ARGUMENTS)

      implicit none

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      integer :: i,j,k
      integer :: nx,ny,nz

      CCTK_REAL :: adm_bs_trA,aux

      CCTK_REAL, PARAMETER :: third = 1.0D0/3.0D0

#include "macro/BSUPPERMET_declare.h"

c     Grid.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

c     Force the lapse to be positive if required

      if (force_lapse_positive .eq. 1) then

        alp = max(alp, force_lapse_minimum)

      end if

c     remove trA.

      do k=2,nz-1
         do j=2,ny-1
            do i=2,nx-1

c              Include macro to calculate inverse conformal metric.

#include "macro/BSUPPERMET_guts.h"

c              Find trace of ADM_BS_Aij.

               adm_bs_trA = BSUPPERMET_UXX*ADM_BS_Axx(i,j,k)
     &                    + BSUPPERMET_UYY*ADM_BS_Ayy(i,j,k)
     &                    + BSUPPERMET_UZZ*ADM_BS_Azz(i,j,k)
     &             + 2.0D0*(BSUPPERMET_UXY*ADM_BS_Axy(i,j,k)
     &                    + BSUPPERMET_UXZ*ADM_BS_Axz(i,j,k)
     &                    + BSUPPERMET_UYZ*ADM_BS_Ayz(i,j,k))

c              Remove trace of ADM_BS_Aij.

               aux = third*adm_bs_trA

               ADM_BS_Axx(i,j,k) = ADM_BS_Axx(i,j,k) - aux*ADM_BS_gxx(i,j,k)
               ADM_BS_Axy(i,j,k) = ADM_BS_Axy(i,j,k) - aux*ADM_BS_gxy(i,j,k)
               ADM_BS_Axz(i,j,k) = ADM_BS_Axz(i,j,k) - aux*ADM_BS_gxz(i,j,k)
               ADM_BS_Ayy(i,j,k) = ADM_BS_Ayy(i,j,k) - aux*ADM_BS_gyy(i,j,k)
               ADM_BS_Ayz(i,j,k) = ADM_BS_Ayz(i,j,k) - aux*ADM_BS_gyz(i,j,k)
               ADM_BS_Azz(i,j,k) = ADM_BS_Azz(i,j,k) - aux*ADM_BS_gzz(i,j,k)

c              Remove determinant of ADM_BS_gij.

               aux = 1.0D0/BSDETG_DETG**third

               ADM_BS_gxx(i,j,k) = aux*ADM_BS_gxx(i,j,k)
               ADM_BS_gxy(i,j,k) = aux*ADM_BS_gxy(i,j,k)
               ADM_BS_gxz(i,j,k) = aux*ADM_BS_gxz(i,j,k)
               ADM_BS_gyy(i,j,k) = aux*ADM_BS_gyy(i,j,k)
               ADM_BS_gyz(i,j,k) = aux*ADM_BS_gyz(i,j,k)
               ADM_BS_gzz(i,j,k) = aux*ADM_BS_gzz(i,j,k)

c              Transfer trA to trK

               ADM_BS_K(i,j,k) = ADM_BS_K(i,j,k) + trAcoeff*adm_bs_trA

            end do
         end do
      end do

#include "macro/BSUPPERMET_undefine.h"

      end subroutine ADM_BSSN_removetrA

