/*@@
  @header   ASOURCES_guts.h
  @date     Nov 98
  @author   Gabrielle Allen
  @desc
  Macro for source term of the BS A evolution equation 
  Eq. 17 of the BS paper.
  @enddesc
@@*/

#ifndef ASOURCES_GUTS
#define ASOURCES_GUTS

#include "BSSTF_guts.h"
#include "CDCDALPHA_guts.h"
#include "NABALPHA_guts.h"
#include "RICCITF_guts.h"
#include "AA_guts.h"


      ASOURCES_ALP = alp(i,j,k)
      ASOURCES_K   = ADM_BS_K(i,j,k)

      ASOURCES_PHIFAC = exp(4*ADM_BS_phi(i,j,k))

      IF (conformal_state == 0) THEN
         ASOURCES_PSIFAC = 1.0d0
      ELSE
         ASOURCES_PSIFAC = psi(i,j,k)**4
      ENDIF

      ASOURCES_EXPTERM = 1.0d0/(ASOURCES_PHIFAC*ASOURCES_PSIFAC)


/* Do the matter source terms */
/* -------------------------- */

      ASOURCES_TEMP = - eightpi*ASOURCES_EXPTERM*ASOURCES_ALP

      ASOURCES_MATTERXX = ASOURCES_TEMP*BSSTF_SXX
      ASOURCES_MATTERXY = ASOURCES_TEMP*BSSTF_SXY
      ASOURCES_MATTERXZ = ASOURCES_TEMP*BSSTF_SXZ
      ASOURCES_MATTERYY = ASOURCES_TEMP*BSSTF_SYY
      ASOURCES_MATTERYZ = ASOURCES_TEMP*BSSTF_SYZ
      ASOURCES_MATTERZZ = ASOURCES_TEMP*BSSTF_SZZ


/* Do the vacuum source terms */
/* -------------------------- */

      ASOURCES_TEMP = third*NABALPHA_NABALPHA_PHYS*ASOURCES_PHIFAC
     &              *ASOURCES_PSIFAC

c     XX component.

#ifdef OPT
      ASOURCES_TERM1 = - CDCDALPHA_CDXXDALPHA_PHYS
     &               + ASOURCES_TEMP*lg(XX,i,j,kc)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RXX 
      ASOURCES_TERM3 = ASOURCES_K*lA(XX,i,j,kc)
      ASOURCES_TERM4 = - 2.0d0*AA_AAXX
#else
      ASOURCES_TERM1 = - CDCDALPHA_CDXXDALPHA_PHYS
     &               + ASOURCES_TEMP*ADM_BS_gxx(i,j,k)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RXX 
      ASOURCES_TERM3 = ASOURCES_K*ADM_BS_AXX(i,j,k)
      ASOURCES_TERM4 = - 2.0d0*AA_AAXX
#endif
      ASOURCES_METRICXX = ASOURCES_EXPTERM*(ASOURCES_TERM1 + ASOURCES_TERM2)
     &                  + ASOURCES_ALP*(ASOURCES_TERM3 + ASOURCES_TERM4)

c     XY component.
#ifdef OPT
      ASOURCES_TERM1 = - CDCDALPHA_CDXYDALPHA_PHYS
     &               + ASOURCES_TEMP*lg(XY,i,j,kc)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RXY 
      ASOURCES_TERM3 = ASOURCES_K*lA(XY,i,j,kc)
      ASOURCES_TERM4 = - 2.0d0*AA_AAXY
#else
      ASOURCES_TERM1 = - CDCDALPHA_CDXYDALPHA_PHYS
     &               + ASOURCES_TEMP*ADM_BS_gxy(i,j,k)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RXY 
      ASOURCES_TERM3 = ASOURCES_K*ADM_BS_AXY(i,j,k)
      ASOURCES_TERM4 = - 2.0d0*AA_AAXY
#endif
      ASOURCES_METRICXY = ASOURCES_EXPTERM*(ASOURCES_TERM1 + ASOURCES_TERM2)
     &                  + ASOURCES_ALP*(ASOURCES_TERM3 + ASOURCES_TERM4)

c     XZ component.
#ifdef OPT
      ASOURCES_TERM1 = - CDCDALPHA_CDXZDALPHA_PHYS
     &               + ASOURCES_TEMP*lg(XZ,i,j,kc)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RXZ 
      ASOURCES_TERM3 = ASOURCES_K*lA(XZ,i,j,kc)
      ASOURCES_TERM4 = - 2.0d0*AA_AAXZ
#else
      ASOURCES_TERM1 = - CDCDALPHA_CDXZDALPHA_PHYS
     &               + ASOURCES_TEMP*ADM_BS_gxz(i,j,k)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RXZ 
      ASOURCES_TERM3 = ASOURCES_K*ADM_BS_AXZ(i,j,k)
      ASOURCES_TERM4 = - 2.0d0*AA_AAXZ
#endif
      ASOURCES_METRICXZ = ASOURCES_EXPTERM*(ASOURCES_TERM1 + ASOURCES_TERM2)
     &                  + ASOURCES_ALP*(ASOURCES_TERM3 + ASOURCES_TERM4)

c     YY component.
#ifdef OPT
      ASOURCES_TERM1 = - CDCDALPHA_CDYYDALPHA_PHYS
     &               + ASOURCES_TEMP*lg(YY,i,j,kc)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RYY 
      ASOURCES_TERM3 = ASOURCES_K*lA(YY,i,j,kc)
      ASOURCES_TERM4 = - 2.0d0*AA_AAYY
#else
      ASOURCES_TERM1 = - CDCDALPHA_CDYYDALPHA_PHYS
     &               + ASOURCES_TEMP*ADM_BS_gyy(i,j,k)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RYY 
      ASOURCES_TERM3 = ASOURCES_K*ADM_BS_AYY(i,j,k)
      ASOURCES_TERM4 = - 2.0d0*AA_AAYY
#endif
      ASOURCES_METRICYY = ASOURCES_EXPTERM*(ASOURCES_TERM1 + ASOURCES_TERM2)
     &                  + ASOURCES_ALP*(ASOURCES_TERM3 + ASOURCES_TERM4)

c     YZ component.
#ifdef OPT
      ASOURCES_TERM1 = - CDCDALPHA_CDYZDALPHA_PHYS
     &               + ASOURCES_TEMP*lg(YZ,i,j,kc)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RYZ 
      ASOURCES_TERM3 = ASOURCES_K*lA(YZ,i,j,kc)
      ASOURCES_TERM4 = - 2.0d0*AA_AAYZ
#else
      ASOURCES_TERM1 = - CDCDALPHA_CDYZDALPHA_PHYS
     &               + ASOURCES_TEMP*ADM_BS_gyz(i,j,k)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RYZ 
      ASOURCES_TERM3 = ASOURCES_K*ADM_BS_AYZ(i,j,k)
      ASOURCES_TERM4 = - 2.0d0*AA_AAYZ
#endif

      ASOURCES_METRICYZ = ASOURCES_EXPTERM*(ASOURCES_TERM1 + ASOURCES_TERM2)
     &                  + ASOURCES_ALP*(ASOURCES_TERM3 + ASOURCES_TERM4)

c     ZZ component.
#ifdef OPT
      ASOURCES_TERM1 = - CDCDALPHA_CDZZDALPHA_PHYS
     &               + ASOURCES_TEMP*lg(ZZ,i,j,kc)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RZZ
      ASOURCES_TERM3 = ASOURCES_K*lA(ZZ,i,j,kc)
      ASOURCES_TERM4 = - 2.0d0*AA_AAZZ
#else
      ASOURCES_TERM1 = - CDCDALPHA_CDZZDALPHA_PHYS
     &               + ASOURCES_TEMP*ADM_BS_gzz(i,j,k)
      ASOURCES_TERM2 = ASOURCES_ALP*RICCITF_RZZ
      ASOURCES_TERM3 = ASOURCES_K*ADM_BS_AZZ(i,j,k)
      ASOURCES_TERM4 = - 2.0d0*AA_AAZZ
#endif
      ASOURCES_METRICZZ = ASOURCES_EXPTERM*(ASOURCES_TERM1 + ASOURCES_TERM2)
     &                  + ASOURCES_ALP*(ASOURCES_TERM3 + ASOURCES_TERM4)

/* Complete source terms */
/* --------------------- */

      adm_bs_sAxx(i,j,k) = ASOURCES_METRICXX + ASOURCES_MATTERXX
      adm_bs_sAxy(i,j,k) = ASOURCES_METRICXY + ASOURCES_MATTERXY
      adm_bs_sAxz(i,j,k) = ASOURCES_METRICXZ + ASOURCES_MATTERXZ
      adm_bs_sAyy(i,j,k) = ASOURCES_METRICYY + ASOURCES_MATTERYY
      adm_bs_sAyz(i,j,k) = ASOURCES_METRICYZ + ASOURCES_MATTERYZ
      adm_bs_sAzz(i,j,k) = ASOURCES_METRICZZ + ASOURCES_MATTERZZ

#endif
