  /*@@
   @file      Whisky_CalcUpdate.F90
   @date      Thu Jan  11 11:03:32 2002
   @author    Ian Hawke
   @desc 
   Calculates the update terms given the fluxes. Moved to here so that 
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"


 /*@@
   @routine    UpdateCalculation 
   @date       Wed Feb 13 11:03:32 2002
   @author     Ian Hawke
   @desc 
   Calculates the update terms from the fluxes.
   @enddesc 
   @calls     
   @calledby   
   @history 
   Moved out of the Riemann solver routines to make the FishEye /
   weighted flux calculation easier.
   @endhistory 

@@*/


subroutine UpdateCalculation(CCTK_ARGUMENTS)
    
  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  CCTK_INT :: i,j,k
  CCTK_REAL :: idx, alp_l, alp_r

!!$  if (flux_direction == 1) then
!!$    idx = 1.d0 / cctk_delta_space(1)
!!$  else if (flux_direction == 2) then
!!$    idx = 1.d0 / cctk_delta_space(2)
!!$  else if (flux_direction == 3) then
!!$    idx = 1.d0 / cctk_delta_space(3)
!!$  else
!!$    call CCTK_Warn(0, "Flux direction not x,y,z")
!!$  end if

  idx = 1.d0 / CCTK_DELTA_SPACE(flux_direction)

  if (use_weighted_fluxes == 0) then
  
    do k = whisky_stencil + 1, cctk_lsh(3) - whisky_stencil
      do j = whisky_stencil + 1, cctk_lsh(2) - whisky_stencil
        do i = whisky_stencil + 1, cctk_lsh(1) - whisky_stencil
          
          alp_l = 0.5d0 * (alp(i,j,k) + &
               alp(i-xoffset,j-yoffset,k-zoffset))
          alp_r = 0.5d0 * (alp(i,j,k) + &
               alp(i+xoffset,j+yoffset,k+zoffset))
          
          densrhs(i,j,k) = densrhs(i,j,k) + &
               (alp_l * densflux(i-xoffset,j-yoffset,k-zoffset) - &
               alp_r * densflux(i,j,k)) * idx
          sxrhs(i,j,k) = sxrhs(i,j,k) + &
               (alp_l * sxflux(i-xoffset,j-yoffset,k-zoffset) - &
               alp_r * sxflux(i,j,k)) * idx
          syrhs(i,j,k) = syrhs(i,j,k) + &
               (alp_l * syflux(i-xoffset,j-yoffset,k-zoffset) - &
               alp_r * syflux(i,j,k)) * idx
          szrhs(i,j,k) = szrhs(i,j,k) + &
               (alp_l * szflux(i-xoffset,j-yoffset,k-zoffset) - &
               alp_r * szflux(i,j,k)) * idx
          taurhs(i,j,k) = taurhs(i,j,k) + &
               (alp_l * tauflux(i-xoffset,j-yoffset,k-zoffset) - &
               alp_r * tauflux(i,j,k)) * idx
          
        enddo
      enddo
    enddo

  else

    call CCTK_WARN(0, "Support for this parameter has been removed")

!!$    do k = whisky_stencil + 1, cctk_lsh(3) - whisky_stencil
!!$      do j = whisky_stencil + 1, cctk_lsh(2) - whisky_stencil
!!$        do i = whisky_stencil + 1, cctk_lsh(1) - whisky_stencil
!!$          
!!$          alp_l = 0.5d0 * (alp(i,j,k) + &
!!$               alp(i-xoffset,j-yoffset,k-zoffset))
!!$          alp_r = 0.5d0 * (alp(i,j,k) + &
!!$               alp(i+xoffset,j+yoffset,k+zoffset))
!!$          
!!$          densrhs(i,j,k) = densrhs(i,j,k) + &
!!$               (alp_l * &
!!$               &cell_surface(i-xoffset,j-yoffset,k-zoffset,flux_direction) * &
!!$               &densflux(i-xoffset,j-yoffset,k-zoffset) - &
!!$               alp_r * &
!!$               &cell_surface(i,j,k,flux_direction) * &
!!$               &densflux(i,j,k)) * idx / cell_volume(i,j,k)
!!$          sxrhs(i,j,k) = sxrhs(i,j,k) + &
!!$               (alp_l * &
!!$               &cell_surface(i-xoffset,j-yoffset,k-zoffset,flux_direction) * &
!!$               &sxflux(i-xoffset,j-yoffset,k-zoffset) - &
!!$               alp_r * &
!!$               &cell_surface(i,j,k,flux_direction) * &
!!$               &sxflux(i,j,k)) * idx / cell_volume(i,j,k)
!!$          syrhs(i,j,k) = syrhs(i,j,k) + &
!!$               (alp_l * &
!!$               &cell_surface(i-xoffset,j-yoffset,k-zoffset,flux_direction) * &
!!$               &syflux(i-xoffset,j-yoffset,k-zoffset) - &
!!$               alp_r * &
!!$               &cell_surface(i,j,k,flux_direction) * &
!!$               &syflux(i,j,k)) * idx / cell_volume(i,j,k)
!!$          szrhs(i,j,k) = szrhs(i,j,k) + &
!!$               (alp_l * &
!!$               &cell_surface(i-xoffset,j-yoffset,k-zoffset,flux_direction) * &
!!$               &szflux(i-xoffset,j-yoffset,k-zoffset) - &
!!$               alp_r * &
!!$               &cell_surface(i,j,k,flux_direction) * &
!!$               &szflux(i,j,k)) * idx / cell_volume(i,j,k)
!!$          taurhs(i,j,k) = taurhs(i,j,k) + &
!!$               (alp_l * &
!!$               &cell_surface(i-xoffset,j-yoffset,k-zoffset,flux_direction) * &
!!$               &tauflux(i-xoffset,j-yoffset,k-zoffset) - &
!!$               alp_r * &
!!$               &cell_surface(i,j,k,flux_direction) * &
!!$               &tauflux(i,j,k)) * idx / cell_volume(i,j,k)
!!$          
!!$        enddo
!!$      enddo
!!$    enddo

  end if
  
  return

end subroutine UpdateCalculation
  
