 /*@@
   @file      Whisky_RegisterVars.F90
   @date      Thu Mar  7 20:00:56 2002
   @author    
   @desc 
   Routine to register the variables with the MoL thorn.
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Functions.h"
#include "cctk_Parameters.h"

subroutine Whisky_Register(CCTK_ARGUMENTS)

  implicit none  

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS
  DECLARE_CCTK_FUNCTIONS

  CCTK_INT :: index1, index2, ierr

!!$  If we''re evolving register as normal...

  ierr = 0

  call CCTK_IsFunctionAliased(ierr, "MoLRegisterEvolvedGroup")
  if (ierr == 0) then
    call CCTK_WARN(0, "The MoLRegisterEvolvedGroup function hasn't been aliased!")
  end if
  call CCTK_IsFunctionAliased(ierr, "MoLRegisterConstrainedGroup")
  if (ierr == 0) then
    call CCTK_WARN(0, "The MoLRegisterConstrainedGroup function hasn't been aliased!")
  end if
  call CCTK_IsFunctionAliased(ierr, "MoLRegisterSaveAndRestoreGroup")
  if (ierr == 0) then
    call CCTK_WARN(0, "The MoLRegisterSaveAndRestoreGroup function hasn't been aliased!")
  end if
  call CCTK_IsFunctionAliased(ierr, "MoLRegisterEvolved")
  if (ierr == 0) then
    call CCTK_WARN(0, "The MoLRegisterEvolved function hasn't been aliased!")
  end if
  call CCTK_IsFunctionAliased(ierr, "MoLRegisterConstrained")
  if (ierr == 0) then
    call CCTK_WARN(0, "The MoLRegisterConstrained function hasn't been aliased!")
  end if

  ierr = 0
  
  if (whisky_evolve .eq. 1) then

    call CCTK_VarIndex(index1, "whisky::dens")
    call CCTK_VarIndex(index2, "whisky::densrhs")
    ierr = MoLRegisterEvolved(index1,index2)
    call CCTK_VarIndex(index1, "whisky::sx")
    call CCTK_VarIndex(index2, "whisky::sxrhs")
    ierr = MoLRegisterEvolved(index1,index2)
    call CCTK_VarIndex(index1, "whisky::sy")
    call CCTK_VarIndex(index2, "whisky::syrhs")
    ierr = MoLRegisterEvolved(index1,index2)
    call CCTK_VarIndex(index1, "whisky::sz")
    call CCTK_VarIndex(index2, "whisky::szrhs")
    ierr = MoLRegisterEvolved(index1,index2)
    
    if (CCTK_EQUALS(whisky_eos_type,"General")) then
      
      call CCTK_VarIndex(index1, "whisky::tau")
      call CCTK_VarIndex(index2, "whisky::taurhs")
      ierr = MoLRegisterEvolved(index1,index2)
      
!!$      call CCTK_GroupIndex(index1, "whisky::whisky_cons_var")
!!$      call CCTK_GroupIndex(index2, "whisky::whisky_rhs_var")
!!$      ierr = MoLRegisterEvolvedGroup(index1,index2)
!!$      
!!$      if (ierr .ne. 0) then
!!$
!!$        call CCTK_WARN(0, "Error registering conserved variables with MoL")
!!$
!!$      end if
      
    else if (CCTK_EQUALS(whisky_eos_type,"Polytype")) then
      
      call CCTK_VarIndex(index1, "whisky::tau")
      ierr = MoLRegisterConstrained(index1)
      
    else
      
      call CCTK_WARN(0, "Don't recognize the type of EOS!")
      
    end if
    
    call CCTK_VarIndex(index1, "whisky::rho")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::velx")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::vely")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::velz")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::press")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::eps")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::w_lorentz")
    ierr = MoLRegisterConstrained(index1)
    
    call CCTK_VarIndex(index1, "ADMBase::alp")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::gxx")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::gxy")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::gxz")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::gyy")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::gyz")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::gzz")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::kxx")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::kxy")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::kxz")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::kyy")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::kyz")
    ierr = MoLRegisterSaveAndRestore(index1)
    call CCTK_VarIndex(index1, "ADMBase::kzz")
    ierr = MoLRegisterSaveAndRestore(index1)
    
    if (.not.CCTK_EQUALS(initial_shift,"none")) then

      if (CCTK_EQUALS(shift_evolution_method,"Comoving")) then
      
        call CCTK_VarIndex(index1, "ADMBase::betax")
        ierr = MoLRegisterConstrained(index1)
        call CCTK_VarIndex(index1, "ADMBase::betay")
        ierr = MoLRegisterConstrained(index1)
        call CCTK_VarIndex(index1, "ADMBase::betaz")
        ierr = MoLRegisterConstrained(index1)

        call CCTK_GroupIndex(index1, "Whisky::whisky_coords")
        call CCTK_GroupIndex(index2, "Whisky::whisky_coords_rhs")
        ierr = MoLRegisterEvolvedGroup(index1, index2)

      else

        call CCTK_VarIndex(index1, "ADMBase::betax")
        ierr = MoLRegisterSaveAndRestore(index1)
        call CCTK_VarIndex(index1, "ADMBase::betay")
        ierr = MoLRegisterSaveAndRestore(index1)
        call CCTK_VarIndex(index1, "ADMBase::betaz")
        ierr = MoLRegisterSaveAndRestore(index1)
      
      end if
      
    end if
    
  else

    call CCTK_VarIndex(index1, "whisky::dens")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::sx")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::sy")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::sz")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::tau")
    ierr = MoLRegisterConstrained(index1)
    
    call CCTK_VarIndex(index1, "whisky::rho")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::velx")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::vely")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::velz")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::press")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::eps")
    ierr = MoLRegisterConstrained(index1)
    call CCTK_VarIndex(index1, "whisky::w_lorentz")
    ierr = MoLRegisterConstrained(index1)
    
  end if

end subroutine Whisky_Register
