 /*@@
   @file      Whisky_UpdateMask.F90
   @date      Wed Mar 13 14:18:38 2002
   @author    
   @desc 
   Alter the update terms if inside the atmosphere or excision region
   @enddesc 
 @@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

 /*@@
   @routine    UpdateMask
   @date       Wed Mar 13 14:18:38 2002
   @author     Ian Hawke, Luca Baiotti
   @desc 
   Alter the update terms if inside the atmosphere or excision region
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine WhiskyUpdateMask(CCTK_ARGUMENTS)

  implicit none
  
  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  CCTK_INT :: i,j,k
  CCTK_REAL :: frac, det

  frac = CCTK_DELTA_TIME

  do k = 1, cctk_lsh(3)
    do j = 1, cctk_lsh(2)
      do i = 1, cctk_lsh(1)

        if ( (atmosphere_mask(i,j,k) .eq. 1) .or. &
             (tau(i,j,k) + frac * taurhs(i,j,k) .le. 0.d0) .or. &
!!$             (dens(i,j,k) + frac * densrhs(i,j,k) .le. &
!!$             whisky_dens_min) ) then
             (dens(i,j,k) + frac * densrhs(i,j,k) .le. &
             0.d0) ) then

          densrhs(i,j,k) = 0.d0
          sxrhs(i,j,k) = 0.d0
          syrhs(i,j,k) = 0.d0
          szrhs(i,j,k) = 0.d0
          taurhs(i,j,k) = 0.d0
          atmosphere_mask(i,j,k) = 1

        end if

      end do
    end do
  end do

end subroutine WhiskyUpdateMask

 /*@@
   @routine    Whisky_SetupMask
   @date       Thu Jun 20 13:27:28 2002
   @author     Ian Hawke
   @desc 
   Initialize the mask to be zero.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine Whisky_SetupMask(CCTK_ARGUMENTS)

  implicit none

  DECLARE_CCTK_ARGUMENTS

  ! Initialize all rhs variables and the mask.
  ! The former vars need to be initialized since there is
  ! no rhs computation in CCTK_INITIAL or POSTINITIAL.                                                       

  densrhs = 0.0d0
  taurhs  = 0.0d0
  sxrhs   = 0.0d0
  syrhs   = 0.0d0
  szrhs   = 0.0d0

  atmosphere_mask = 0

end subroutine Whisky_SetupMask

 /*@@
   @routine    Whisky_AtmosphereReset
   @date       Thu Jun 20 13:30:51 2002
   @author     Ian Hawke
   @desc 
   After MoL has evolved, if a point is supposed to be reset then do so.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

subroutine Whisky_AtmosphereReset(CCTK_ARGUMENTS)

  implicit none

  DECLARE_CCTK_ARGUMENTS
  DECLARE_CCTK_PARAMETERS

  CCTK_INT :: i, j, k
  CCTK_REAL :: det, psi4pt
  
  do k = 1, cctk_lsh(3)
    do j = 1, cctk_lsh(2)
      do i = 1, cctk_lsh(1)

        if (atmosphere_mask(i, j, k) .eq. 1) then

          rho(i,j,k) = whisky_rho_min
          velx(i,j,k) = 0.d0
          vely(i,j,k) = 0.d0
          velz(i,j,k) = 0.d0
          if (conformal_state .eq. 0) then
            call SpatialDeterminant(gxx(i,j,k), gxy(i,j,k), gxz(i,j,k), &
                 gyy(i,j,k), gyz(i,j,k), gzz(i,j,k), det)
            call prim2conpolytype(whisky_polytrope_handle, &
                 gxx(i,j,k), gxy(i,j,k), gxz(i,j,k), &
                 gyy(i,j,k), gyz(i,j,k), gzz(i,j,k), det, &
                 dens(i,j,k), sx(i,j,k), sy(i,j,k), sz(i,j,k), &
                 tau(i,j,k), rho(i,j,k), velx(i,j,k), vely(i,j,k), &
                 velz(i,j,k), eps(i,j,k), press(i,j,k), w_lorentz(i,j,k))
          else
            psi4pt = psi(i,j,k)**4
            call SpatialDeterminant(psi4pt*gxx(i,j,k), psi4pt*gxy(i,j,k), &
                 psi4pt*gxz(i,j,k), psi4pt*gyy(i,j,k), psi4pt*gyz(i,j,k), &
                 psi4pt*gzz(i,j,k), det)
            call prim2conpolytype(whisky_polytrope_handle, &
                 psi4pt*gxx(i,j,k), psi4pt*gxy(i,j,k), psi4pt*gxz(i,j,k), &
                 psi4pt*gyy(i,j,k), psi4pt*gyz(i,j,k), psi4pt*gzz(i,j,k), &
                 det, dens(i,j,k), sx(i,j,k), sy(i,j,k), sz(i,j,k), &
                 tau(i,j,k), rho(i,j,k), velx(i,j,k), vely(i,j,k), &
                 velz(i,j,k), eps(i,j,k), press(i,j,k), w_lorentz(i,j,k))
          end if
          atmosphere_mask(i, j, k) = 0
        end if

      end do
    end do
  end do

!!$  call Whisky_Boundaries(CCTK_PASS_FTOF)

end subroutine Whisky_AtmosphereReset
