 /*@@
   @file      rnsid_rfr.c
   @date      Tue May  5 11:34:52 1998
   @author    Tom Goodale
   @desc 
   Register the rnsid routines with the rfr, and provide an interface to them.
   @enddesc 
   @version $Header: /EUNetwork/EUHydro/Whisky_RNSID/src/rnsid_rfr.c,v 1.32 2005/04/30 07:53:25 baiotti Exp $
 @@*/

/* Changed by N. Stergioulas, 24/10/2001: include latest source files
from cactus 3 */

#include "cctk.h"
#include "cctk_Arguments.h"
#include "cctk_Parameters.h"

#include "Boundary.h"
#include "Symmetry.h"

#include "FishEye.h"

#include "cctk_DefineThorn.h" 

#include "rnsid.h"

static const char *rcsid="$Header: /EUNetwork/EUHydro/Whisky_RNSID/src/rnsid_rfr.c,v 1.32 2005/04/30 07:53:25 baiotti Exp $";
CCTK_FILEVERSION(EUHydro_Whisky_RNSID_rnsid_rfr_c)

 /*@@
   @routine    rnsid_init
   @date       Tue May  5 11:37:37 1998
   @author     Tom Goodale
   @desc 
   Calls the rnsid routines.
   @enddesc 
   @calls     
   @calledby   
   @history 
 
   @endhistory 

@@*/

void rnsid_init(CCTK_ARGUMENTS);

void rnsid_init(CCTK_ARGUMENTS)

{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;
  
  CCTK_REAL  Omega,                    /* angular velocity */
    R_e,                               /* circumferential radius */
    r_e,                               /* coordinate radius */
    mass0;                             /* rest mass */
    
  CCTK_REAL eos_k, eos_ideal_fluid_gamma, atm_factor;

  CCTK_REAL *x_coord=0, *y_coord=0, *z_coord=0;

  CCTK_REAL max_rho, max_eps, max_rho_tmp, max_eps_tmp;

  CCTK_REAL gamma_center;  /* central value of 3-determinant of metric,
                              needed for applying perturbations */

  int i, j, k, handle, ierr, Reduction_Handle;
  int sw[3];

#include "EOS_Base.h"

  handle = EOS_Handle("2D_Polytrope");
  if (handle < 0) 
  {
    CCTK_WARN(0,"For this hack you need to compile with EOS_Polytrope");
  }

  eos_k = RNS_K < 0.0 ? EOS_Pressure(handle, 1.0, 1.0) : RNS_K;
  eos_ideal_fluid_gamma = RNS_Gamma < 0.0 ? 1.0 + eos_k / 
    EOS_SpecificIntEnergy(handle,1.0,1.0) : RNS_Gamma;

  /* Find correct atmosphere for initial data*/

  if (rho_abs_min > 0.0)
  {
    atm_factor = rho_abs_min;
  }
  else
  {
    atm_factor = whisky_rho_central * rho_rel_min;
  }

  if (initial_rho_abs_min > 0.0)
  {
    atm_factor = initial_rho_abs_min;
  }
  else if (initial_rho_rel_min > 0.0)
  {
    atm_factor = whisky_rho_central * initial_rho_rel_min;
  }

  if (initial_atmosphere_factor > 0.0)
  {
    atm_factor *= initial_atmosphere_factor;
  }

  /* SET-UP INITIAL DATA */
  
  if (use_fisheye)
    {
      x_coord = (CCTK_REAL*)CCTK_VarDataPtr(cctkGH, 0, "FishEye::fex");
      y_coord = (CCTK_REAL*)CCTK_VarDataPtr(cctkGH, 0, "FishEye::fey");
      z_coord = (CCTK_REAL*)CCTK_VarDataPtr(cctkGH, 0, "FishEye::fez");
    }
  else
    {
      x_coord = x;
      y_coord = y;
      z_coord = z;
    }
  
  rnsid(cctkGH, x_coord, y_coord, z_coord,eos_k, eos_ideal_fluid_gamma, atm_factor, &Omega, &R_e, &r_e, &mass0, &gamma_center);  
  
  if (use_fisheye)
    {
      for (i = 0; 
           i < cctkGH->cctk_lsh[0]*cctkGH->cctk_lsh[1]*cctkGH->cctk_lsh[2];
           i++) 
	{
#ifdef FISHEYE_ACTIVE
	  StdToFish_ddsym(x[i], y[i], z[i], 
			  gxx + i, gxy + i,gxz + i,
			  gyy + i, gyz + i, gzz + i);
	  StdToFish_ddsym(x[i], y[i], z[i], 
			  kxx + i, kxy + i, kxz + i,
			  kyy + i, kyz + i, kzz + i);
	  StdToFish_u_Vec(x[i], y[i], z[i], 
			  velx + i, vely + i, velz + i);
	  StdToFish_u_Vec(x[i], y[i], z[i], 
			  betax + i, betay + i, betaz + i);
#endif
	}
    }
  
  for (i=0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
    {
      CCTK_REAL det, uxx, uxy, uxz, uyy, uyz, uzz;
      
      SpatialDet(gxx[i],gxy[i],gxz[i],gyy[i],gyz[i],gzz[i],&det);
      UpperMet(&uxx,&uxy,&uxz,&uyy,&uyz,&uzz,det,
               gxx[i],gxy[i],gxz[i],gyy[i],gyz[i],gzz[i]);
      Con2PrimPoly(handle,dens+i,sx+i,sy+i,sz+i,tau+i,
                   rho+i,velx+i,vely+i,velz+i,eps+i,press+i,w_lorentz+i,
                   uxx,uxy,uxz,uyy,uyz,uzz,det,x[i],y[i],z[i],r[i],
                   atm_factor);
    }      
  
  /* Apply boundary conditions */
  
  i = 0;
  sw[0] = whisky_stencil;
  sw[1] = whisky_stencil;
  sw[2] = whisky_stencil;
  i += CartSymGN(cctkGH, "Whisky::Whisky_cons_scalar_var");
  i += CartSymGN(cctkGH, "Whisky::Whisky_prim_scalar_var");
  i += CartSymGN(cctkGH, "Whisky::Whisky_cons_vector_var");
  i += CartSymGN(cctkGH, "Whisky::Whisky_prim_vector_var");
  i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_cons_scalar_var");
  i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_prim_scalar_var");
  i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_cons_vector_var");
  i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_prim_vector_var");
  if (i != 0) 
    {
      CCTK_WARN(0, "Failure calling the boundaries from RNSID.");
    }
  
  
  /* ADD A PERTURBATION */
  
  if (perturbation) 
    {
      
      rnsid_pert(cctkGH, x_coord, y_coord, z_coord, eos_k, eos_ideal_fluid_gamma, Omega, R_e, r_e, mass0, gamma_center); 

      /* Apply boundary conditions */
      
      i = 0;
      sw[0] = whisky_stencil;
      sw[1] = whisky_stencil;
      sw[2] = whisky_stencil;
      i += CartSymGN(cctkGH, "Whisky::Whisky_cons_scalar_var");
      i += CartSymGN(cctkGH, "Whisky::Whisky_prim_scalar_var");
      i += CartSymGN(cctkGH, "Whisky::Whisky_cons_vector_var");
      i += CartSymGN(cctkGH, "Whisky::Whisky_prim_vector_var");
      i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_cons_scalar_var");
      i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_prim_scalar_var");
      i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_cons_vector_var");
      i += BndFlatGN(cctkGH, sw, "Whisky::Whisky_prim_vector_var");
      if (i != 0) 
	{
	  CCTK_WARN(0, "Failure calling the boundaries from RNSID.");
	}
      
    } 

  /* Set consistent conservative variables */

  if (consistent_boundaries)
    {
      
      for (i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	{
	  CCTK_REAL det;
          
	  SpatialDet(gxx[i],gxy[i],gxz[i],
		     gyy[i],gyz[i],gzz[i],&det);
	  Prim2ConPoly(handle,gxx[i],gxy[i],gxz[i],
		       gyy[i],gyz[i],gzz[i],det,
		       dens+i,sx+i,sy+i,sz+i,tau+i,
		       rho[i],velx[i],vely[i],velz[i],
		       eps[i],press[i],w_lorentz[i]);
	}
    }
  else 
    {  
      for (k = whisky_stencil - 1; k < cctk_lsh[2] - whisky_stencil + 1; k++)
	{
	  for (j = whisky_stencil - 1; j < cctk_lsh[1] - whisky_stencil + 1; j++)
	    {
	      for (i = whisky_stencil - 1; i < cctk_lsh[0] - whisky_stencil + 1; i++)
		{
		  CCTK_REAL det;
		  CCTK_INT idx = CCTK_GFINDEX3D(cctkGH,i,j,k);
		  
		  SpatialDet(gxx[idx],gxy[idx],gxz[idx],
			     gyy[idx],gyz[idx],gzz[idx],&det);
		  Prim2ConPoly(handle,gxx[idx],gxy[idx],gxz[idx],
			       gyy[idx],gyz[idx],gzz[idx],det,
			       dens+idx,sx+idx,sy+idx,sz+idx,tau+idx,
			       rho[idx],velx[idx],vely[idx],velz[idx],
			       eps[idx],press[idx],w_lorentz[idx]);
		}
	    }
	}
    }
  
  if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::metric") > 1)
    {
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	{
	  gxx_p[i] = gxx[i];
	  gyy_p[i] = gyy[i];
	  gzz_p[i] = gzz[i];
	  gxy_p[i] = gxy[i];
	  gxz_p[i] = gxz[i];
	  gyz_p[i] = gyz[i];
	}
      if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::metric") > 2)
	{
	  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	    {
	      gxx_p_p[i] = gxx[i];
	      gyy_p_p[i] = gyy[i];
	      gzz_p_p[i] = gzz[i];
	      gxy_p_p[i] = gxy[i];
	      gxz_p_p[i] = gxz[i];
	      gyz_p_p[i] = gyz[i];
	    }
	}
    }
  
  if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::curv") > 1)
    {
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	{
	  kxx_p[i] = kxx[i];
	  kyy_p[i] = kyy[i];
	  kzz_p[i] = kzz[i];
	  kxy_p[i] = kxy[i];
	  kxz_p[i] = kxz[i];
	  kyz_p[i] = kyz[i];
	}
      if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::curv") > 2)
	{
	  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	    {
	      kxx_p_p[i] = kxx[i];
	      kyy_p_p[i] = kyy[i];
	      kzz_p_p[i] = kzz[i];
	      kxy_p_p[i] = kxy[i];
	      kxz_p_p[i] = kxz[i];
	      kyz_p_p[i] = kyz[i];
	    }
	}
    }
  
  if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::lapse") > 1)
    {
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	{
	  alp_p[i] = alp[i];
	}
      if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::lapse") > 2)
	{
	  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	    {
	      alp_p_p[i] = alp[i];
	    }
	}
    }
  
  if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::shift") > 1)
    {
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	{
	  betax_p[i] = betax[i];
	  betay_p[i] = betay[i];
	  betaz_p[i] = betaz[i];
	}
      if (CCTK_ActiveTimeLevels(cctkGH, "ADMBase::shift") > 2)
	{
	  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	    {
	      betax_p_p[i] = betax[i];
	      betay_p_p[i] = betay[i];
	      betaz_p_p[i] = betaz[i];
	    }
	}
    }
  
  if (CCTK_ActiveTimeLevels(cctkGH, "Whisky::whisky_prim_scalar_var") > 1)
    {
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	{
	  rho_p[i] = rho[i];
	  velx_p[i] = velx[i];
	  vely_p[i] = vely[i];
	  velz_p[i] = velz[i];
	  press_p[i] = press[i];
	  eps_p[i] = eps[i];
	  w_lorentz_p[i] = w_lorentz[i];
	}
      if (CCTK_ActiveTimeLevels(cctkGH, "Whisky::whisky_prim_scalar_var") > 2)
	{
	  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	    {
	      rho_p_p[i] = rho[i];
	      velx_p_p[i] = velx[i];
	      vely_p_p[i] = vely[i];
	      velz_p_p[i] = velz[i];
	      press_p_p[i] = press[i];
	      eps_p_p[i] = eps[i];
	      w_lorentz_p_p[i] = w_lorentz[i];
	    }
	}
    }
  
  if (CCTK_ActiveTimeLevels(cctkGH, "Whisky::whisky_cons_scalar_var") > 1)
    {
      for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	{
	  dens_p[i] = dens[i];
	  sx_p[i] = sx[i];
	  sy_p[i] = sy[i];
	  sz_p[i] = sz[i];
	  tau_p[i] = tau[i];
	}
      if (CCTK_ActiveTimeLevels(cctkGH, "Whisky::whisky_cons_scalar_var") > 2)
	{
	  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
	    {
	      dens_p_p[i] = dens[i];
	      sx_p_p[i] = sx[i];
	      sy_p_p[i] = sy[i];
	      sz_p_p[i] = sz[i];
	      tau_p_p[i] = tau[i];
	    }
	}
    }
  
  /* Find correct atmosphere for evolution. In general, rho_central != rho_max  */
  
  /* initialize max_rho */
  max_rho = 0.0;
  max_eps = 0.0;
  
  /* Find the processor-local maximum of rho and eps */
  for(i = 0; i < cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]; i++)
    {
      if (rho[i] > max_rho)
	{  
	  max_rho = rho[i];
	}
      
      if (eps[i] > max_eps)
	{	   
	  max_eps = eps[i];
	}  
    }

  /* Find the global maximum of rho and eps */

  Reduction_Handle = CCTK_ReductionHandle("maximum");
    
  max_rho_tmp = max_rho;
  max_eps_tmp = max_eps;

  ierr = CCTK_ReduceLocScalar(cctkGH, -1, Reduction_Handle, &max_rho_tmp, &max_rho, CCTK_VARIABLE_REAL);

  if (ierr != 0)
    {
      CCTK_WARN(0, "Failed to communicate the value of rho_max from all procesors");
    }
  
  ierr = CCTK_ReduceLocScalar(cctkGH, -1, Reduction_Handle, &max_eps_tmp, &max_eps, CCTK_VARIABLE_REAL);
  
  if (ierr != 0)
    {
      CCTK_WARN(0, "Failed to communicate the value of eps_max from all procesors");
    }
  /* From now on, max_rho and eps_max are the maximum values over all the grid, not only locally */

  if (rho_abs_min > 0.0)
    {
      *whisky_rho_min = rho_abs_min;
    }
  else
    {
      *whisky_rho_min = max_rho * rho_rel_min;
    }
  

  return;
}

